import { isString } from "../../libs/types.mjs";
import { CAUSE, ValueSchemaError } from "../../libs/ValueSchemaError.mjs";
export var CHECKSUM_ALGORITHM;

(function (CHECKSUM_ALGORITHM) {
  /** Luhn algorithm; used in credit card and IMEI (also known as MOD-10 algorithm) */
  CHECKSUM_ALGORITHM["LUHN"] = "luhn";
  /** checksum as credit card; equivalent to LUHN */

  CHECKSUM_ALGORITHM["CREDIT_CARD"] = "luhn";
  /** modulus10/weight3:1 algorithm; used in ISBN13, EAN and JAN */

  CHECKSUM_ALGORITHM["MODULUS10_WEIGHT3_1"] = "modulus10/weight3:1";
  /** ISBN-13; equivalent to MODULUS10_WEIGHT3_1 */

  CHECKSUM_ALGORITHM["ISBN13"] = "modulus10/weight3:1";
  /** EAN code; equivalent to MODULUS10_WEIGHT3_1 */

  CHECKSUM_ALGORITHM["EAN"] = "modulus10/weight3:1";
  /** JAN code; equivalent to MODULUS10_WEIGHT3_1 */

  CHECKSUM_ALGORITHM["JAN"] = "modulus10/weight3:1";
})(CHECKSUM_ALGORITHM || (CHECKSUM_ALGORITHM = {}));
/**
 * apply schema
 * @param values input/output values
 * @param options options
 * @param keyStack key stack for error handling
 * @returns applied value
 */


export function applyTo(values, options, keyStack) {
  if (options.checksum === undefined) {
    return false;
  } // istanbul ignore next


  if (!isString(values.output)) {
    return false;
  }

  if (check(values.output, options.checksum)) {
    return false;
  }

  return ValueSchemaError.raise(CAUSE.CHECKSUM, values, keyStack);
}
/**
 * check string
 * @param value value to check
 * @param algorithm check algorithm
 * @returns OK/NG
 */

function check(value, algorithm) {
  switch (algorithm) {
    case CHECKSUM_ALGORITHM.LUHN:
      return checkLuhn(value);

    case CHECKSUM_ALGORITHM.MODULUS10_WEIGHT3_1:
      return checkModulus10Weight31(value);

    default:
      return false;
  }
}
/**
 * check by Luhn algorithm (used by credit card)
 * @param value value to check
 * @returns OK/NG
 */


function checkLuhn(value) {
  const {
    length
  } = value;
  let sum = 0;

  for (let index = length - 1; index >= 0; index -= 2) {
    // odd columns
    sum += Number(value[index]);
  }

  for (let index = length - 2; index >= 0; index -= 2) {
    // even columns
    const num = Number(value[index]) * 2;
    sum += num % 10;
    sum += Math.floor(num / 10);
  }

  return sum % 10 === 0;
}
/**
 * check by Modulus 10 / Weight 3:1 algorithm (used by ISBN/EAN/JAN)
 * @param value value to check
 * @returns OK/NG
 */


function checkModulus10Weight31(value) {
  const {
    length
  } = value;
  let sum = 0;

  for (let index = 0; index < length - 1; index += 2) {
    sum += Number(value[index]);
  }

  for (let index = 1; index < length - 1; index += 2) {
    sum += Number(value[index]) * 3;
  }

  const mod = sum % 10;
  return (10 - mod) % 10 === Number(value[length - 1]);
}